/*
 * Copyright (c) 2002-2015, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */

package fr.paris.lutece.plugins.appointment.modules.solr.service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.solr.client.solrj.SolrServerException;
import org.apache.solr.client.solrj.response.UpdateResponse;
import org.apache.solr.client.solrj.util.ClientUtils;

import fr.paris.lutece.plugins.appointment.business.form.Form;
import fr.paris.lutece.plugins.appointment.business.slot.Slot;
import fr.paris.lutece.plugins.appointment.service.FormService;
import fr.paris.lutece.plugins.appointment.web.dto.AppointmentFormDTO;
import fr.paris.lutece.plugins.search.solr.business.SolrServerService;
import fr.paris.lutece.plugins.search.solr.business.field.Field;
import fr.paris.lutece.plugins.search.solr.indexer.SolrIndexer;
import fr.paris.lutece.plugins.search.solr.indexer.SolrIndexerService;
import fr.paris.lutece.plugins.search.solr.indexer.SolrItem;
import fr.paris.lutece.portal.service.search.SearchItem;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

/**
 * Indexer of the slots and forms of RDV V2
 * 
 * @author Laurent Payen
 *
 */
public class SolrAppointmentIndexer implements SolrIndexer
{

    public static final String BEAN_NAME = "appointment-solr.solrAppointmentIndexer";

    @Override
    public List<String> indexDocuments( )
    {
        List<String> errors = new ArrayList<String>( );
        for ( AppointmentFormDTO appointmentForm : FormService.buildAllActiveAppointmentForm( ) )
        {
            try
            {
                synchronized( appointmentForm )
                {
                    writeFormAndListSlots( appointmentForm );
                }
            }
            catch( IOException e )
            {
                AppLogService.error( "Error indexing AppointmentForm" + appointmentForm.getIdForm( ), e );
                errors.add( e.toString( ) );
            }
        }
        return errors;
    }

    @Override
    public String getResourceUid( String strResourceId, String strResourceType )
    {
        StringBuilder stringBuilder = new StringBuilder( strResourceId );
        if ( Utilities.RESOURCE_TYPE_SLOT.equals( strResourceType ) )
        {
            stringBuilder.append( '_' ).append( Utilities.SHORT_NAME_SLOT );
        }
        else
            if ( Utilities.RESOURCE_TYPE_APPOINTMENT.equals( strResourceType ) )
            {
                stringBuilder.append( '_' ).append( Utilities.SHORT_NAME_APPOINTMENT );
            }
            else
            {
                AppLogService.error( "SolrAppointmentIndexer, unknown resourceType: " + strResourceType );
                return null;
            }
        return stringBuilder.toString( );
    }

    @Override
    public List<Field> getAdditionalFields( )
    {
        return new ArrayList<Field>( );
    }

    @Override
    public String getDescription( )
    {
        return Utilities.APPOINTMENT_DESCRIPTION;
    }

    @Override
    public List<SolrItem> getDocuments( String arg0 )
    {
        return new ArrayList<SolrItem>( );
    }

    @Override
    public String getName( )
    {
        return Utilities.APPOINTMENT_FORM_NAME;
    }

    @Override
    public List<String> getResourcesName( )
    {
        return new ArrayList<String>( );
    }

    @Override
    public String getVersion( )
    {
        return Utilities.APPOINTMENT_VERSION;
    }

    @Override
    public boolean isEnable( )
    {
        return Boolean.valueOf( AppPropertiesService.getProperty( Utilities.PROPERTY_INDEXER_ENABLE ) );
    }

    /**
     * Write the Appointment Form and all the slots of this form to Solr
     * 
     * @param appointmentForm
     *            the appointment form
     * @throws CorruptIndexException
     * @throws IOException
     */
    public void writeFormAndListSlots( AppointmentFormDTO appointmentForm ) throws CorruptIndexException, IOException
    {
        writeFormAndListSlots( appointmentForm, SolrIndexerService.getSbLogs( ) );
    }

    /**
     * Write the Appointment Form and all the related slots to Solr
     * 
     * @param appointmentForm
     *            the Appointment Form
     * @param sbLogs
     *            the logs
     * @throws CorruptIndexException
     * @throws IOException
     */
    public void writeFormAndListSlots( AppointmentFormDTO appointmentForm, StringBuffer sbLogs ) throws CorruptIndexException, IOException
    {
        synchronized( appointmentForm )
        {
            List<Slot> listAllSlots = SlotUtil.getAllSlots( appointmentForm );
            SolrIndexerService.write( FormUtil.getFormItem( appointmentForm, listAllSlots ), sbLogs );
            List<SolrItem> listItems = new ArrayList<>( );
            for ( Slot appointmentSlot : listAllSlots )
            {
                listItems.add( SlotUtil.getSlotItem( appointmentForm, appointmentSlot ) );
            }
            SolrIndexerService.write( listItems, sbLogs );
        }
    }

    /**
     * Write / Update the slot and then the related form (for the number of available places) to Solr
     * 
     * @param nIdSlot
     *            The id of the slot to write / update
     * @throws CorruptIndexException
     * @throws IOException
     */
    public void writeSlotAndForm( Slot slot ) throws CorruptIndexException, IOException
    {
        writeSlotAndForm( slot, SolrIndexerService.getSbLogs( ) );
    }

    /**
     * Write / update the slot and the related form (for the number of available places) in solr
     * 
     * @param nIdSlot
     *            The id of the slot
     * @param sbLogs
     *            the logs
     * @throws CorruptIndexException
     * @throws IOException
     */
    public void writeSlotAndForm( Slot slot, StringBuffer sbLogs ) throws CorruptIndexException, IOException
    {
        synchronized( slot )
        {
            AppointmentFormDTO appointmentForm = FormService.buildAppointmentForm( slot.getIdForm( ), 0, 0 );
            SolrIndexerService.write( SlotUtil.getSlotItem( appointmentForm, slot ), sbLogs );
            List<Slot> listAllSlots = SlotUtil.getAllSlots( appointmentForm );
            SolrIndexerService.write( FormUtil.getFormItem( appointmentForm, listAllSlots ), sbLogs );
        }
    }

    /**
     * Delete the Appointment Form and all the related slots in Solr
     * 
     * @param nIdForm
     *            The id of the Form
     * @param sbLogs
     *            the logs
     * @throws SolrServerException
     * @throws IOException
     */
    public void deleteFormAndListSlots( int nIdForm, StringBuffer sbLogs ) throws SolrServerException, IOException
    {
        Form form = FormService.findFormLightByPrimaryKey( nIdForm );
        synchronized( form )
        {
            // Remove all indexed values of this site
            StringBuffer sbAppointmentFormUidEscaped = new StringBuffer( ClientUtils.escapeQueryChars( SolrIndexerService.getWebAppName( ) ) );
            sbAppointmentFormUidEscaped.append( Utilities.UNDERSCORE ).append(
                    getResourceUid( Integer.toString( nIdForm ), Utilities.RESOURCE_TYPE_APPOINTMENT ) );
            StringBuffer sbQuery = new StringBuffer( SearchItem.FIELD_UID ).append( ":" ).append( sbAppointmentFormUidEscaped ).append( " OR uid_form_string:" )
                    .append( sbAppointmentFormUidEscaped );
            sbLogs.append( "Delete by query: " ).append( sbQuery ).append( StringUtils.CR ).append( StringUtils.LF );
            UpdateResponse update = SolrServerService.getInstance( ).getSolrServer( ).deleteByQuery( sbQuery.toString( ), 1000 );
            sbLogs.append( "Server response: " ).append( update ).append( StringUtils.CR ).append( StringUtils.LF );
        }
    }

    /**
     * Delete the slot in solr
     * 
     * @param slot
     *            The slot to delete
     * @param sbLogs
     *            the logs
     * @throws SolrServerException
     * @throws IOException
     */
    public void deleteSlot( Slot slot, StringBuffer sbLogs ) throws SolrServerException, IOException
    {
        synchronized( slot )
        {
            StringBuffer sbSlotUidEscaped = new StringBuffer( ClientUtils.escapeQueryChars( SolrIndexerService.getWebAppName( ) ) ).append(
                    Utilities.UNDERSCORE ).append( getResourceUid( SlotUtil.getSlotUid( slot ), Utilities.RESOURCE_TYPE_SLOT ) );
            StringBuffer sbQuery = new StringBuffer( SearchItem.FIELD_UID ).append( ":" ).append( sbSlotUidEscaped );
            sbLogs.append( "Delete by query: " ).append( sbQuery ).append( StringUtils.CR ).append( StringUtils.LF );
            UpdateResponse update = SolrServerService.getInstance( ).getSolrServer( ).deleteByQuery( sbQuery.toString( ), 1000 );
            sbLogs.append( "Server response: " ).append( update ).append( StringUtils.CR ).append( StringUtils.LF );
        }
    }

}
